@@ -154,7 +154,7 @@ class Command(BaseCommand):
             for req in group_deps:
                 req.specifier = get_specifier("")
 
-        reqs = [r for g, deps in all_dependencies.items() if lock_groups is None or g in lock_groups for r in deps]
+        reqs = [r for g, deps in all_dependencies.items() for r in deps if lock_groups is None or g in lock_groups]
         with hooks.skipping("post_lock"):
             resolved = do_lock(
                 project,
